@@ -1,45 +1,39 @@
 use bevy::prelude::*;
+use bevy_inspector_egui::RegisterInspectable;
 use bevy_inspector_egui::{widgets::ResourceInspector, Inspectable, InspectorPlugin};
-use bevy_inspector_egui::{InspectableRegistry, WorldInspectorParams, WorldInspectorPlugin};
+use bevy_inspector_egui::{WorldInspectorParams, WorldInspectorPlugin};
 
 fn main() {
-    let mut app = App::build();
-
-    app.insert_resource(Msaa { samples: 4 })
+    App::new()
+        .insert_resource(Msaa { samples: 4 })
         .add_plugins(DefaultPlugins)
         .insert_resource(WorldInspectorParams {
             despawnable_entities: true,
+            highlight_changes: true,
             ..Default::default()
         })
         .add_plugin(WorldInspectorPlugin::new())
         .add_plugin(InspectorPlugin::<Resources>::new())
         .register_type::<MyReflectedComponent>()
-        .add_startup_system(setup.system());
-
-    // getting registry from world
-    let mut registry = app
-        .world_mut()
-        .get_resource_or_insert_with(InspectableRegistry::default);
-
-    // registering custom component to be able to edit it in inspector
-    registry.register::<MyInspectableComponent>();
-
-    app.run();
+        .register_inspectable::<MyInspectableComponent>()
+        .add_startup_system(setup.system())
+        .add_system(movement.system())
+        .run();
 }
 
 #[derive(Inspectable, Default)]
 struct Resources {
-    ambient_light: ResourceInspector<bevy::pbr::AmbientLight>,
+    ambient_light: ResourceInspector<AmbientLight>,
     clear_color: ResourceInspector<ClearColor>,
 }
 
-#[derive(Inspectable, Default)]
+#[derive(Component, Inspectable, Default)]
 pub struct MyInspectableComponent {
     foo: f32,
     bar: usize,
 }
 
-#[derive(Reflect, Default)]
+#[derive(Component, Reflect, Default)]
 #[reflect(Component)]
 pub struct MyReflectedComponent {
     str: String,
@@ -52,6 +46,14 @@ fn setup(
     mut meshes: ResMut<Assets<Mesh>>,
     mut materials: ResMut<Assets<StandardMaterial>>,
 ) {
+    commands.spawn_bundle((
+        MyInspectableComponent::default(),
+        MyReflectedComponent {
+            str: "str".to_string(),
+            list: vec![2.0],
+        },
+        Name::new("Custom components"),
+    ));
     commands
         .spawn_bundle(PerspectiveCameraBundle {
             transform: Transform::from_matrix(Mat4::face_toward(
@@ -68,11 +70,6 @@ fn setup(
             material: materials.add(Color::rgb_u8(80, 233, 54).into()),
             ..Default::default()
         })
-        .insert(MyInspectableComponent::default())
-        .insert(MyReflectedComponent {
-            str: "str".to_string(),
-            list: vec![2.0],
-        })
         .insert(Name::new("Floor"));
     commands
         .spawn_bundle(PbrBundle {
@@ -90,6 +87,7 @@ fn setup(
                     material: materials.add(Color::rgb(1.0, 1.0, 1.0).into()),
                     ..Default::default()
                 })
+                .insert(TeleportTarget)
                 .insert(Name::new("Child"))
                 .with_children(|commands| {
                     commands
@@ -112,11 +110,12 @@ fn setup(
             material: materials.add(Color::RED.into()),
             ..Default::default()
         })
+        .insert(RotateTarget)
         .insert(Name::new("Sphere"));
     commands
-        .spawn_bundle(LightBundle {
+        .spawn_bundle(PointLightBundle {
             transform: Transform::from_xyz(10.3, 8.0, -2.3),
-            light: Light {
+            point_light: PointLight {
                 range: 20.0,
                 intensity: 1237.0,
                 ..Default::default()
@@ -125,9 +124,9 @@ fn setup(
         })
         .insert(Name::new("Light"));
     commands
-        .spawn_bundle(LightBundle {
+        .spawn_bundle(PointLightBundle {
             transform: Transform::from_xyz(-6.2, 8.0, 4.3),
-            light: Light {
+            point_light: PointLight {
                 range: 20.0,
                 intensity: 245.0,
                 ..Default::default()
@@ -136,3 +135,49 @@ fn setup(
         })
         .insert(Name::new("Second Light"));
 }
+
+#[derive(Component)]
+struct RotateTarget;
+#[derive(Component)]
+struct TeleportTarget;
+
+struct TeleportState {
+    timer: Timer,
+    count: u32,
+}
+
+impl Default for TeleportState {
+    fn default() -> Self {
+        Self {
+            timer: Timer::from_seconds(1.0, true),
+            count: 0,
+        }
+    }
+}
+
+/// simple movement in the scene
+#[allow(clippy::type_complexity)]
+fn movement(
+    time: Res<Time>,
+    mut state: Local<TeleportState>,
+    mut qs: QuerySet<(
+        QueryState<&mut Transform, With<RotateTarget>>,
+        QueryState<&mut Transform, With<TeleportTarget>>,
+    )>,
+) {
+    for mut transform in qs.q0().iter_mut() {
+        // rotate around vertical axis through origin
+        transform.translation =
+            Quat::from_axis_angle(Vec3::Y, time.delta_seconds()) * transform.translation;
+    }
+    if state.timer.tick(time.delta()).just_finished() {
+        for mut transform in qs.q1().iter_mut() {
+            // jump to new position to the left or right
+            transform.translation += match state.count % 4 {
+                0 | 3 => -Vec3::X,
+                _ => Vec3::X,
+            };
+        }
+        state.count += 1;
+    }
+}
