@@ -58,7 +58,7 @@ impl<T> DerefMut for ReflectedUI<T> {
 impl<T: Reflect> Inspectable for ReflectedUI<T> {
     type Attributes = ();
 
-    fn ui(&mut self, ui: &mut egui::Ui, _: Self::Attributes, context: &Context) -> bool {
+    fn ui(&mut self, ui: &mut egui::Ui, _: Self::Attributes, context: &mut Context) -> bool {
         ui_for_reflect(&mut self.0, ui, context)
     }
 }
@@ -79,12 +79,18 @@ macro_rules! try_downcast_ui {
 ///
 /// This function gets used for the implementation of [`Inspectable`](crate::Inspectable)
 /// for [`ReflectedUI`](ReflectedUI).
-pub fn ui_for_reflect(value: &mut dyn Reflect, ui: &mut egui::Ui, context: &Context) -> bool {
-    if let Some(world) = unsafe { context.world() } {
-        if let Some(inspect_registry) = world.get_resource::<InspectableRegistry>() {
-            if let Ok(changed) = inspect_registry.try_execute(value, ui, context) {
+pub fn ui_for_reflect(value: &mut dyn Reflect, ui: &mut egui::Ui, context: &mut Context) -> bool {
+    if let Some((cx, world)) = context.take_world() {
+        if world.contains_resource::<InspectableRegistry>() {
+            let res =
+                world.resource_scope(|world, inspectable_registry: Mut<InspectableRegistry>| {
+                    let mut context = cx.with_world(world);
+                    inspectable_registry.try_execute(value, ui, &mut context)
+                });
+            if let Ok(changed) = res {
                 return changed;
             }
+            context.world = Some(world);
         }
     }
 
@@ -98,7 +104,7 @@ pub fn ui_for_reflect(value: &mut dyn Reflect, ui: &mut egui::Ui, context: &Cont
     }
 }
 
-fn ui_for_reflect_struct(value: &mut dyn Struct, ui: &mut egui::Ui, context: &Context) -> bool {
+fn ui_for_reflect_struct(value: &mut dyn Struct, ui: &mut egui::Ui, context: &mut Context) -> bool {
     let mut changed = false;
     ui.vertical_centered(|ui| {
         let grid = Grid::new(value.type_id());
@@ -109,7 +115,7 @@ fn ui_for_reflect_struct(value: &mut dyn Struct, ui: &mut egui::Ui, context: &Co
                     None => ui.label("<missing>"),
                 };
                 if let Some(field) = value.field_at_mut(i) {
-                    changed |= ui_for_reflect(field, ui, &context.with_id(i as u64));
+                    changed |= ui_for_reflect(field, ui, &mut context.with_id(i as u64));
                 } else {
                     ui.label("<missing>");
                 }
@@ -120,14 +126,18 @@ fn ui_for_reflect_struct(value: &mut dyn Struct, ui: &mut egui::Ui, context: &Co
     changed
 }
 
-fn ui_for_tuple_struct(value: &mut dyn TupleStruct, ui: &mut egui::Ui, context: &Context) -> bool {
+fn ui_for_tuple_struct(
+    value: &mut dyn TupleStruct,
+    ui: &mut egui::Ui,
+    context: &mut Context,
+) -> bool {
     let mut changed = false;
     let grid = Grid::new(value.type_id());
     grid.show(ui, |ui| {
         for i in 0..value.field_len() {
             ui.label(i.to_string());
             if let Some(field) = value.field_mut(i) {
-                changed |= ui_for_reflect(field, ui, &context.with_id(i as u64));
+                changed |= ui_for_reflect(field, ui, &mut context.with_id(i as u64));
             } else {
                 ui.label("<missing>");
             }
@@ -137,14 +147,14 @@ fn ui_for_tuple_struct(value: &mut dyn TupleStruct, ui: &mut egui::Ui, context:
     changed
 }
 
-fn ui_for_tuple(value: &mut dyn Tuple, ui: &mut egui::Ui, context: &Context) -> bool {
+fn ui_for_tuple(value: &mut dyn Tuple, ui: &mut egui::Ui, context: &mut Context) -> bool {
     let mut changed = false;
     let grid = Grid::new(value.type_id());
     grid.show(ui, |ui| {
         for i in 0..value.field_len() {
             ui.label(i.to_string());
             if let Some(field) = value.field_mut(i) {
-                changed |= ui_for_reflect(field, ui, &context.with_id(i as u64));
+                changed |= ui_for_reflect(field, ui, &mut context.with_id(i as u64));
             } else {
                 ui.label("<missing>");
             }
@@ -154,7 +164,7 @@ fn ui_for_tuple(value: &mut dyn Tuple, ui: &mut egui::Ui, context: &Context) ->
     changed
 }
 
-fn ui_for_list(list: &mut dyn List, ui: &mut egui::Ui, context: &Context) -> bool {
+fn ui_for_list(list: &mut dyn List, ui: &mut egui::Ui, context: &mut Context) -> bool {
     let mut changed = false;
 
     ui.vertical(|ui| {
@@ -167,7 +177,7 @@ fn ui_for_list(list: &mut dyn List, ui: &mut egui::Ui, context: &Context) -> boo
                 /*if utils::ui::label_button(ui, "✖", egui::Color32::RED) {
                     to_delete = Some(i);
                 }*/
-                changed |= ui_for_reflect(val, ui, &context.with_id(i as u64));
+                changed |= ui_for_reflect(val, ui, &mut context.with_id(i as u64));
             });
 
             if i != len - 1 {
@@ -199,7 +209,7 @@ fn ui_for_map(_value: &mut dyn Map, ui: &mut egui::Ui) -> bool {
     false
 }
 
-fn ui_for_reflect_value(value: &mut dyn Reflect, ui: &mut egui::Ui, context: &Context) -> bool {
+fn ui_for_reflect_value(value: &mut dyn Reflect, ui: &mut egui::Ui, context: &mut Context) -> bool {
     try_downcast_ui!(
         value ui context =>
         f32, f64, u8, u16, u32, u64, i8, i16, i32, i64,
