@@ -2,20 +2,19 @@ pub(crate) mod impls;
 mod inspectable_registry;
 mod plugin;
 
-use bevy::{render::camera::Camera, window::WindowId};
+use bevy::{ecs::archetype::Archetype, render::camera::Camera, ui::FocusPolicy, window::WindowId};
 pub use inspectable_registry::InspectableRegistry;
 pub use plugin::WorldInspectorPlugin;
 
 use bevy::{
     ecs::{
-        component::{ComponentId, ComponentInfo, ComponentTicks, StorageType},
+        component::{ComponentId, ComponentTicks, StorageType},
         entity::EntityLocation,
         query::{FilterFetch, WorldQuery},
         world::EntityRef,
     },
     prelude::*,
     reflect::{TypeRegistryArc, TypeRegistryInternal},
-    render::render_graph::base::MainPass,
     utils::HashSet,
 };
 use bevy_egui::egui::{self, Color32};
@@ -23,10 +22,7 @@ use egui::CollapsingHeader;
 use pretty_type_name::pretty_type_name_str;
 use std::{any::TypeId, borrow::Cow, cell::Cell};
 
-use crate::{
-    utils::{sort_iter_if, ui::label_button},
-    Context,
-};
+use crate::{utils::ui::label_button, Context};
 use impls::EntityAttributes;
 use inspectable_registry::InspectCallback;
 
@@ -45,6 +41,10 @@ pub struct WorldInspectorParams {
     pub despawnable_entities: bool,
     /// The window the inspector should be displayed on
     pub window: WindowId,
+    /// Filter entities by name.
+    pub name_filter: Option<String>,
+    /// Highlight when components are changed in the world
+    pub highlight_changes: bool,
 }
 
 impl WorldInspectorParams {
@@ -56,6 +56,8 @@ impl WorldInspectorParams {
             enabled: true,
             despawnable_entities: false,
             window: WindowId::primary(),
+            name_filter: Some(String::new()),
+            highlight_changes: false,
         }
     }
 
@@ -64,6 +66,11 @@ impl WorldInspectorParams {
         self.ignore_components.insert(TypeId::of::<T>());
     }
 
+    /// Filter entities by name.
+    pub fn filter_by_name<S: Into<String>>(&mut self, filter: S) {
+        self.name_filter = Some(filter.into());
+    }
+
     fn should_ignore_component(&self, type_id: TypeId) -> bool {
         self.ignore_components.contains(&type_id)
     }
@@ -88,9 +95,6 @@ impl Default for WorldInspectorParams {
             TypeId::of::<Children>(),
             TypeId::of::<Parent>(),
             TypeId::of::<PreviousParent>(),
-            TypeId::of::<MainPass>(),
-            TypeId::of::<Draw>(),
-            TypeId::of::<RenderPipelines>(),
         ]
         .iter()
         .copied()
@@ -146,7 +150,7 @@ impl<'a> WorldUIContext<'a> {
     }
 
     /// Displays the world inspector UI.
-    pub fn world_ui<F>(&mut self, ui: &mut egui::Ui, params: &WorldInspectorParams) -> bool
+    pub fn world_ui<F>(&mut self, ui: &mut egui::Ui, params: &mut WorldInspectorParams) -> bool
     where
         F: WorldQuery,
         F::Fetch: FilterFetch,
@@ -159,72 +163,89 @@ impl<'a> WorldUIContext<'a> {
 
         let mut changed = false;
 
-        for entity in root_entities.iter(self.world) {
+        if let Some(filter) = params.name_filter.as_mut() {
+            ui.horizontal(|ui| {
+                ui.label("Named");
+                ui.text_edit_singleline(filter);
+            });
+        }
+
+        let entites = root_entities.iter(self.world).collect::<Vec<_>>();
+        for entity in entites {
             changed |= self.entity_ui(ui, entity, params, dummy_id.with(entity), &entity_options);
         }
 
         changed
     }
 
-    fn entity_ui(
-        &self,
+    /// Displays an entity and its children in a collapsing header.
+    pub fn entity_ui(
+        &mut self,
         ui: &mut egui::Ui,
         entity: Entity,
         params: &WorldInspectorParams,
         id: egui::Id,
         entity_options: &EntityAttributes,
     ) -> bool {
-        CollapsingHeader::new(self.entity_name(entity))
-            .id_source(id.with(entity))
-            .show(ui, |ui| {
-                self.entity_ui_inner(ui, entity, params, id, entity_options)
+        let entity_name = self.entity_name(entity);
+        let show_entity = params
+            .name_filter
+            .as_ref()
+            .map(|filter| {
+                filter.is_empty() || entity_name.to_lowercase().contains(&filter.to_lowercase())
             })
-            .body_returned
-            .unwrap_or(false)
+            .unwrap_or(true);
+
+        if show_entity {
+            CollapsingHeader::new(self.entity_name(entity).to_string())
+                .id_source(id.with(entity))
+                .show(ui, |ui| {
+                    self.entity_ui_inner(ui, entity, params, id, entity_options)
+                })
+                .body_returned
+                .unwrap_or(false)
+        } else {
+            let children = self.world.get::<Children>(entity).cloned();
+            if let Some(children) = children {
+                for &child in children.iter() {
+                    self.entity_ui(ui, child, params, id.with(child), entity_options);
+                }
+            }
+            false
+        }
     }
 
-    fn entity_ui_inner(
-        &self,
+    /// Displays an entity and its children.
+    pub fn entity_ui_inner(
+        &mut self,
         ui: &mut egui::Ui,
         entity: Entity,
         params: &WorldInspectorParams,
         id: egui::Id,
         entity_options: &EntityAttributes,
     ) -> bool {
-        let entity_ref = match self.world.get_entity(entity) {
-            Some(entity_ref) => entity_ref,
-            None => {
-                ui.label("Entity does not exist");
-                return false;
-            }
-        };
-        let entity_location = entity_ref.location();
-        let archetype = entity_ref.archetype();
-
         let mut changed = false;
 
         changed |= self.component_kind_ui(
             ui,
-            archetype.table_components(),
+            |archetype| archetype.table_components(),
             "Components",
             entity,
-            entity_location,
             params,
             id,
         );
         changed |= self.component_kind_ui(
             ui,
-            archetype.sparse_set_components(),
+            |archetype| archetype.sparse_set_components(),
             "Components (Sparse)",
             entity,
-            entity_location,
             params,
             id,
         );
 
         ui.separator();
 
-        let children = self.world.get::<Children>(entity);
+        let children = self.world.get::<Children>(entity).cloned();
         if let Some(children) = children {
             ui.label("Children");
             for &child in children.iter() {
@@ -244,37 +265,69 @@ impl<'a> WorldUIContext<'a> {
         changed
     }
 
+    /// Safety:
+    /// `entity_location` must point to a valid archetype and index.
     fn component_kind_ui(
-        &self,
+        &mut self,
         ui: &mut egui::Ui,
-        components: &[ComponentId],
+        components: impl Fn(&Archetype) -> &[ComponentId],
         title: &str,
         entity: Entity,
-        entity_location: EntityLocation,
         params: &WorldInspectorParams,
         id: egui::Id,
     ) -> bool {
+        let (entity_location, components) = {
+            let entity_ref = match self.world.get_entity(entity) {
+                Some(entity_ref) => entity_ref,
+                None => {
+                    ui.label("Entity does not exist");
+                    return false;
+                }
+            };
+            let entity_location = entity_ref.location();
+            let archetype = entity_ref.archetype();
+
+            let components = components(archetype).to_vec();
+            (entity_location, components)
+        };
+
         if !components.is_empty() {
             ui.label(title);
 
-            let iter = components.iter().map(|component_id| {
-                let component_info = self.world.components().get_info(*component_id).unwrap();
-                let name = pretty_type_name_str(component_info.name());
-                (name, component_info)
-            });
-            let iter = sort_iter_if(iter, params.sort_components, |a, b| a.0.cmp(&b.0));
+            let mut components: Vec<_> = components
+                .iter()
+                .map(|component_id| {
+                    let component_info = self.world.components().get_info(*component_id).unwrap();
+                    let name = pretty_type_name_str(component_info.name());
+                    (
+                        name,
+                        component_info.id(),
+                        component_info.type_id(),
+                        component_info.layout().size(),
+                    )
+                })
+                .collect();
+
+            if params.sort_components {
+                components.sort_by(|a, b| a.0.cmp(&b.0));
+            }
 
             let mut changed = false;
-            for (name, component_info) in iter {
-                changed |= self.component_ui(
-                    ui,
-                    name,
-                    entity,
-                    entity_location,
-                    component_info,
-                    params,
-                    id,
-                );
+            for (name, component_id, component_type_id, size) in components {
+                let is_zst = size == 0;
+                changed |= unsafe {
+                    self.component_ui(
+                        ui,
+                        name,
+                        entity,
+                        entity_location,
+                        component_id,
+                        component_type_id,
+                        is_zst,
+                        params,
+                        id,
+                    )
+                }
             }
             changed
         } else {
@@ -282,17 +335,21 @@ impl<'a> WorldUIContext<'a> {
         }
     }
 
-    fn component_ui(
-        &self,
+    /// Safety:
+    /// `entity_location` must point to a valid archetype and index.
+    unsafe fn component_ui(
+        &mut self,
         ui: &mut egui::Ui,
         name: String,
         entity: Entity,
         entity_location: EntityLocation,
-        component_info: &ComponentInfo,
+        component_id: ComponentId,
+        component_type_id: Option<TypeId>,
+        is_zst: bool,
         params: &WorldInspectorParams,
         id: egui::Id,
     ) -> bool {
-        let type_id = match component_info.type_id() {
+        let type_id = match component_type_id {
             Some(id) => id,
             None => {
                 ui.label("No type id");
@@ -304,46 +361,83 @@ impl<'a> WorldUIContext<'a> {
             return false;
         }
 
-        let inspectable_registry = self.world.get_resource::<InspectableRegistry>().unwrap();
-        let type_registry = self.world.get_resource::<TypeRegistryArc>().unwrap();
-        let type_registry = &*type_registry.internal.read();
+        // Safety: according to this function's contract, entity_location is valid,
+        // and self.world gives us exclusive access.
+        let (component_ptr, component_ticks) = {
+            let (ptr, ticks) =
+                get_component_and_ticks(self.world, component_id, entity, entity_location).unwrap();
+            (ptr, { &mut *ticks })
+        };
 
-        let id = id.with(component_info.id());
-        CollapsingHeader::new(name)
+        if params.highlight_changes
+            && component_ticks
+                .is_changed(self.world.last_change_tick(), self.world.read_change_tick())
+        {
+            ui.style_mut().visuals.collapsing_header_frame = true;
+            ui.style_mut().visuals.widgets.inactive.bg_stroke = egui::Stroke {
+                width: 1.0,
+                color: egui::Color32::GOLD,
+            };
+            ui.style_mut().visuals.widgets.active.bg_stroke = egui::Stroke {
+                width: 1.0,
+                color: egui::Color32::GOLD,
+            };
+            ui.style_mut().visuals.widgets.hovered.bg_stroke = egui::Stroke {
+                width: 1.0,
+                color: egui::Color32::GOLD,
+            };
+            ui.style_mut().visuals.widgets.noninteractive.bg_stroke = egui::Stroke {
+                width: 1.0,
+                color: egui::Color32::GOLD,
+            };
+        }
+
+        let id = id.with(component_id);
+        let changed = CollapsingHeader::new(name)
             .id_source(id)
             .show(ui, |ui| {
+                ui.reset_style();
+
                 if params.is_read_only(type_id) {
                     ui.set_enabled(false);
                 }
 
-                let world_ptr = self.world as *const _ as *mut _;
-                let context = unsafe {
-                    let id = id_to_u64(&id);
-                    Context::new_ptr(self.ui_ctx, world_ptr).with_id(id)
-                };
+                let result = self.world
+                    .resource_scope(|world, inspectable_registry: Mut<InspectableRegistry>| {
+                        world.resource_scope(|world, type_registry: Mut<TypeRegistryArc>| {
+                        let type_registry = &*type_registry.internal.read();
 
-                let result = unsafe {
-                    try_display(
-                        &self.world,
-                        entity,
-                        entity_location,
-                        component_info,
-                        type_id,
-                        inspectable_registry,
-                        type_registry,
-                        ui,
-                        &context,
-                    )
-                };
+                        // Safety: according to this function's contract, entity_location (and therefore component_ptr) are valid
+                        try_display(
+                            world,
+                            entity,
+                            component_ptr,
+                            is_zst,
+                            type_id,
+                            &*inspectable_registry,
+                            type_registry,
+                            ui,
+                            self.ui_ctx,
+                            id,
+                        )
+                    })
+                });
 
                 if result.is_err() {
-                    ui.label("Inspectable has not been defined for this component");
+                    ui.label("This component is neither `Reflect` nor `Inspectable`.\nMake sure to also call `app.register_type` or `app.register_inspectable`.");
                 }
 
                 result.unwrap_or(false)
             })
             .body_returned
-            .unwrap_or(false)
+            .unwrap_or(false);
+
+        if changed {
+            component_ticks.set_changed(self.world.change_tick());
+        }
+
+        ui.reset_style();
+        changed
     }
 
     fn entity_name(&self, entity: Entity) -> Cow<'_, str> {
@@ -355,72 +449,61 @@ impl<'a> WorldUIContext<'a> {
 }
 
 /// Safety:
-/// The `location` must point to a valid archetype and index,
-/// and the function must have unique access to the components.
+/// `component_ptr` must point to a valid component, and the function must have unique access.
 pub(crate) unsafe fn try_display(
-    world: &World,
+    world: &mut World,
     entity: Entity,
-    location: EntityLocation,
-    component_info: &ComponentInfo,
+    component_ptr: *mut u8,
+    is_zst: bool,
     type_id: TypeId,
     inspectable_registry: &InspectableRegistry,
     type_registry: &TypeRegistryInternal,
     ui: &mut egui::Ui,
-    context: &Context,
+    ui_ctx: Option<&egui::CtxRef>,
+    id: egui::Id,
 ) -> Result<bool, ()> {
     if let Some(inspect_callback) = inspectable_registry.impls.get(&type_id) {
-        let changed = display_by_inspectable_registry(
-            inspect_callback,
-            world,
-            location,
-            entity,
-            component_info,
-            ui,
-            context,
-        );
+        let id = id_to_u64(&id);
+        let mut context = Context::new_world_access(ui_ctx, world);
+        let mut context = context.with_id(id);
+
+        let changed =
+            display_by_inspectable_registry(inspect_callback, component_ptr, ui, &mut context);
         return Ok(changed);
     }
 
-    if component_info.layout().size() == 0 {
+    if is_zst {
         return Ok(false);
     }
 
-    if let Ok(changed) = display_by_reflection(type_registry, type_id, world, entity, ui, context) {
+    if let Ok(changed) =
+        display_by_reflection(type_registry, type_id, world, entity, ui, ui_ctx, id)
+    {
         return Ok(changed);
     }
 
     Err(())
 }
 
+/// Safety:
+/// `component_ptr` must point to a valid component to which the the caller has unique access
 unsafe fn display_by_inspectable_registry(
     inspect_callback: &InspectCallback,
-    world: &World,
-    location: EntityLocation,
-    entity: Entity,
-    component_info: &ComponentInfo,
+    component_ptr: *mut u8,
     ui: &mut egui::Ui,
-    context: &Context,
+    context: &mut Context,
 ) -> bool {
-    let (ptr, ticks) =
-        get_component_and_ticks(world, component_info.id(), entity, location).unwrap();
-    let ticks = { &mut *ticks };
-
-    let changed = inspect_callback(ptr, ui, &context);
-
-    if changed {
-        ticks.set_changed(world.read_change_tick());
-    }
-
-    changed
+    inspect_callback(component_ptr, ui, context)
 }
 
 fn display_by_reflection(
     type_registry: &TypeRegistryInternal,
     type_id: TypeId,
-    world: &World,
+    world: &mut World,
     entity: Entity,
     ui: &mut egui::Ui,
-    context: &Context,
+    ui_ctx: Option<&egui::CtxRef>,
+    id: egui::Id,
 ) -> Result<bool, ()> {
     let registration = type_registry.get(type_id).ok_or(())?;
 
@@ -436,22 +519,28 @@ fn display_by_reflection(
         }
     };
 
-    let mut reflected = unsafe {
+    let mut reflected = {
+        let world = unsafe { &mut *(world as *mut _) };
         reflect_component
-            .reflect_component_unchecked_mut(world, entity)
+            .reflect_component_mut(world, entity)
             .ok_or(())?
     };
+
+    let id = id_to_u64(&id);
+    let mut context = Context::new_world_access(ui_ctx, world);
+    let mut context = context.with_id(id);
+
     Ok(crate::reflect::ui_for_reflect(
         &mut *reflected,
         ui,
-        &context,
+        &mut context,
     ))
 }
 
 // copied from bevy
 #[inline]
 unsafe fn get_component_and_ticks(
-    world: &World,
+    world: &mut World,
     component_id: ComponentId,
     entity: Entity,
     location: EntityLocation,
@@ -465,8 +554,8 @@ unsafe fn get_component_and_ticks(
             let table_row = archetype.entity_table_row(location.index);
             // SAFE: archetypes only store valid table_rows and the stored component type is T
             Some((
-                components.get_unchecked(table_row),
-                components.get_ticks_unchecked(table_row),
+                components.get_data_unchecked(table_row),
+                components.get_ticks_mut_ptr_unchecked(table_row),
             ))
         }
         StorageType::SparseSet => world
@@ -477,10 +566,10 @@ unsafe fn get_component_and_ticks(
     }
 }
 
-fn entity_is_bundle<B: Bundle>(e: &EntityRef) -> bool {
-    B::type_info()
-        .iter()
-        .all(|type_info| e.contains_type_id(type_info.type_id()))
+macro_rules! is_bundle {
+    ($entity:ident: $($ty:ty),* $(,)?) => {
+        $( $entity.contains::<$ty>() && )* true
+    };
 }
 
 fn guess_entity_name(entity: EntityRef) -> Cow<'_, str> {
@@ -495,20 +584,54 @@ fn guess_entity_name(entity: EntityRef) -> Cow<'_, str> {
         }
     }
 
-    if entity_is_bundle::<LightBundle>(&entity) {
+    if is_bundle!(entity: PointLight, Transform, GlobalTransform) {
         return format!("Light ({:?})", entity.id().id()).into();
     }
-    if entity_is_bundle::<TextBundle>(&entity) {
+
+    if is_bundle!(entity: DirectionalLight, Transform, GlobalTransform) {
+        return format!("Directional Light ({:?})", entity.id().id()).into();
+    }
+
+    if is_bundle!(
+        entity: Handle<Mesh>,
+        Handle<StandardMaterial>,
+        Transform,
+        GlobalTransform
+    ) {
+        return format!("Pbr Mesh ({:?})", entity.id().id()).into();
+    }
+
+    if is_bundle!(
+        entity: Node,
+        Style,
+        Text,
+        CalculatedSize,
+        FocusPolicy,
+        Transform,
+        GlobalTransform
+    ) {
         return format!("Entity {:?} (Text)", entity.id().id()).into();
     }
-    if entity_is_bundle::<Text2dBundle>(&entity) {
+    if is_bundle!(
+        entity: Text,
+        Transform,
+        GlobalTransform,
+        bevy::text::Text2dSize
+    ) {
         return format!("Entity {:?} (Text2d)", entity.id().id()).into();
     }
-    if entity_is_bundle::<NodeBundle>(&entity) {
+    if is_bundle!(
+        entity: Node,
+        Style,
+        UiColor,
+        UiImage,
+        Transform,
+        GlobalTransform
+    ) {
         return format!("Entity {:?} (Node)", entity.id().id()).into();
     }
 
-    format!("Entity {}", entity.id().id()).into()
+    format!("Entity {:?}", entity.id()).into()
 }
 
 fn id_to_u64(id: &egui::Id) -> u64 {
